#! /usr/bin/env perl
use Config;
use Sys::Hostname;
use Getopt::Long;
use File::Spec;
use Cwd;
use Time::localtime;

my ( $platform, $taskhook ) = ( 'NOT DEFINED', 'NOT DEFINED' );
my $fail;

GetOptions(
	'taskhook=s' => \$taskhook,
	'fail'       => \$fail,
);
my @args  = @ARGV;
my %table = (
	configure => \&configure,
	build     => \&build,
	test      => \&test,
	scan      => \&scan,
	remote_declare => \&generate_tasklist_nmi,
);

# NMI Macros expected
# NMI_SRCDIR -- subdirectory containing source code
# NMI_ARGS_<taskname> -- arguments for a task
# NMI_ARGS_<taskname>_<platform> -- arguments for a task running on a particular platform
# NMI_TASKLIST -- Lists of remote tasks to run format = "taskname1,taskname2,..."
# NMI_TIMEOUT_<taskname> -- timeout value for a task
# NMI_TIMEOUT_<taskname>_<platform> -- timeout value for a task running on a particular platform
# _NMI_TASKNAME -- Used as the taskname if defined otherwise the value of --taskhook is used

my $srcdir   = $ENV{'NMI_SRCDIR'};
my $startdir = cwd();
my $platform = $ENV{'NMI_PLATFORM'};

# Figure out the name of the task
my $name = $taskhook;
$name = $ENV{'_NMI_TASKNAME'}
  if defined $ENV{'_NMI_TASKNAME'};

# Look up the function in the table not that the table keys are
# prefixes so that multiple calls can be made with different
# arguments.  $found is used to handle the case where the task name is not recognized

my $return;
my $found = 0;
for my $k ( keys %table ) {
	if ( $name =~ m!^\s*$k! ) {
		$return = &{ $table{$k} }($name);
		$found++;
		last;
	}
}

die "Task $name not found" unless $found;
die "Task $name has failed" if $return;
exit 0;

sub configure {
	my ($taskhook) = @_;
	my $args = assemble($taskhook,"args");
	my $command = "./configure $args";
	print "Executing [$command] in $srcdir\n";
	chdir $srcdir;
	my $return = system($command);
	chdir $startdir;
	return $return
}

sub scan {
	print "NMI Settings\n";
	for my $e ( sort keys %ENV ) {
		next unless $e =~ m!NMI_!;
		print "\t\t$e=" . $ENV{$e} . "\n";
	}

}

sub build {
	my ($taskhook) = @_;
	my $args = assemble($taskhook,"args");
	my $command = "make $args";
	print "Executing [$command] in $srcdir\n";
	chdir $srcdir;
	my $return = system($command);
	chdir $startdir;
	return $return
}

sub test {
	my ($taskhook) = @_;
	my $args = assemble($taskhook,"args");
	chdir $srcdir;
	my $command = "make $args check";
	print "Executing [$command] in $srcdir\n";
	my $return = system($command);
	chdir $startdir;
	return $return
}

sub generate_tasklist_nmi {
	my ($taskhook) = @_;
	my @tasklist = split /,/, $ENV{'NMI_TASKLIST'};
	
	open LIST, ">tasklist.nmi";
	print "Generating tasklist.nmi for $taskhook\n";
	for my $l (@tasklist) {
		my $time = assemble($l,"timeout");
		$time = 1 unless defined $time;
		print LIST "$l $time\n";
		print "$l $time\n";
	}
	close LIST;
	return 0;
}

sub generate_results_file {
	my ($taskhook) = @_;
	print "Creating results.tar.gz file for $taskhook\n";
	if ( -e 'tasklist.nmi' ) {
		print "Including tasklist.nmi\n";
		system("tar czvf results.tar.gz $subdir tasklist.nmi");
	}
	else {
		system("tar czvf results.tar.gz $subdir ");
	}
}

sub assemble {
		my ($taskhook,$type) = @_;
		%assemble_types = 
			(
				args=>'NMI_ARGS_',
				timeout => 'NMI_TIMEOUT_',
			);
			my $prefix = $assemble_types{$type};
			die "[$type] not recognized" unless defined $prefix;
	my $args = $ENV{($prefix . $taskhook)};
	$args = "" unless defined $args;
	if (defined $ENV{($prefix . $taskhook . "_" . $platform)}) {
		$args .= " " . $ENV{($prefix . $taskhook . "_" . $platform)};
	}
	return $args;
}